"""Defines classes useful for solving general problems."""

class CaselessDict(dict):
    """Implements a dictionary which ignores case in its keys."""

    def __contains__(self, key):
        return dict.__contains__(self, key.lower())

    def __delitem__(self, key):
        return dict.__delitem__(self, key.lower())

    def __getitem__(self, key):
        return dict.__getitem__(self, key.lower())

    def __setitem__(self, key, value):
        dict.__setitem__(self, key.lower(), value)

    def get(self, key):
        return dict.get(self, key.lower())

    def has_key(self, key):
        return dict.has_key(self, key.lower())


class DictArray:
    """Implements a multi-dimensional array which is indexed at each level by
       a dictionary, allowing for quick access at each level."""

    def __init__(self, numLevels):
        self.numLevels = numLevels
        self.topLevel = {}

    def __GetSubDict(self, args):
        """Return dictionary for a particular level in the array."""
        if len(args) >= self.numLevels:
            raise "Expecting less than %d arguments" % self.numLevels
        mapping = self.topLevel
        for arg in args:
            if arg not in mapping:
                return {}
            mapping = mapping[arg]
        return mapping

    def __SetValue(self, keys, value):
        """Set a value in the array."""
        if len(keys) != self.numLevels:
            raise "Can only set a value at the leaf level directly."
        mapping = self.topLevel
        for key in keys[:-1]:
            if key not in mapping:
                mapping[key] = {}
            mapping = mapping[key]
        mapping[keys[-1]] = value

    def AppendValue(self, *args):
        """Add an element to the list which is the value in the array."""
        keys = args[:-1]
        array = self.GetValue(*keys)
        if array is None:
            array = []
            self.__SetValue(keys, array)
        array.append(args[-1])

    def Exists(self, *args):
        """Return a boolean indicating if the item exists in the array."""
        if len(args) != self.numLevels:
            raise "Expecting %d arguments" % self.numLevels
        mapping = self.topLevel
        for arg in args:
            if arg not in mapping:
                return False
            mapping = mapping[arg]
        return True

    def GetValue(self, *args):
        """Return the value at the given level of the tree."""
        if len(args) != self.numLevels:
            raise "Expecting %d arguments" % self.numLevels
        value = self.topLevel
        for arg in args:
            if arg not in value:
                return None
            value = value[arg]
        return value

    def HasValues(self):
        """Return a boolean indicating if any values are in the array."""
        return len(self.topLevel) != 0

    def SetValue(self, *args):
        """Sets the value in the array."""
        self.__SetValue(args[:-1], args[-1])

    def SortedKeys(self, *args):
        """Return the keys at the specified level."""
        keys = self.__GetSubDict(args).keys()
        keys.sort()
        return keys

    def Split(self, *args):
        """Split the array at the given level and return a new array rooted
           at that level."""
        mapping = self.__GetSubDict(args)
        array = DictArray(self.numLevels - len(args))
        array.topLevel = mapping
        return array

